@@ -23,14 +23,14 @@
 
 #include <bitset.h>
 #include <bitset/stats.h>
+#include <closeout.h>
 #include <configmake.h>
 #include <progname.h>
+#include <quote.h>
 #include <quotearg.h>
 #include <relocatable.h> /* relocate2 */
 #include <timevar.h>
 
-#include "LR0.h"
-#include "closeout.h"
 #include "complain.h"
 #include "conflicts.h"
 #include "derives.h"
@@ -38,15 +38,15 @@
 #include "fixits.h"
 #include "getargs.h"
 #include "gram.h"
-#include "lalr.h"
 #include "ielr.h"
+#include "lalr.h"
+#include "lr0.h"
 #include "muscle-tab.h"
 #include "nullable.h"
 #include "output.h"
-#include "print.h"
-#include "print_graph.h"
+#include "print-graph.h"
 #include "print-xml.h"
-#include <quote.h>
+#include "print.h"
 #include "reader.h"
 #include "reduce.h"
 #include "scan-code.h"
@@ -66,11 +66,12 @@ main (int argc, char *argv[])
   {
     char *cp = NULL;
     char const *localedir = relocate2 (LOCALEDIR, &cp);
-    (void) bindtextdomain (PACKAGE, localedir);
-    (void) bindtextdomain ("bison-runtime", localedir);
+    bindtextdomain ("bison", localedir);
+    bindtextdomain ("bison-gnulib", localedir);
+    bindtextdomain ("bison-runtime", localedir);
     free (cp);
   }
-  (void) textdomain (PACKAGE);
+  textdomain ("bison");
 
   {
     char const *cp = getenv ("LC_CTYPE");
@@ -155,31 +156,37 @@ main (int argc, char *argv[])
 
   print_precedence_warnings ();
 
-  /* Output file names. */
-  compute_output_file_names ();
+  /* Whether to generate output files.  */
+  bool generate = !(feature_flag & feature_syntax_only);
 
-  /* Output the detailed report on the grammar.  */
-  if (report_flag)
+  if (generate)
     {
-      timevar_push (tv_report);
-      print_results ();
-      timevar_pop (tv_report);
-    }
+      /* Output file names. */
+      compute_output_file_names ();
 
-  /* Output the graph.  */
-  if (graph_flag)
-    {
-      timevar_push (tv_graph);
-      print_graph ();
-      timevar_pop (tv_graph);
-    }
+      /* Output the detailed report on the grammar.  */
+      if (report_flag)
+        {
+          timevar_push (tv_report);
+          print_results ();
+          timevar_pop (tv_report);
+        }
 
-  /* Output xml.  */
-  if (xml_flag)
-    {
-      timevar_push (tv_xml);
-      print_xml ();
-      timevar_pop (tv_xml);
+      /* Output the graph.  */
+      if (graph_flag)
+        {
+          timevar_push (tv_graph);
+          print_graph ();
+          timevar_pop (tv_graph);
+        }
+
+      /* Output xml.  */
+      if (xml_flag)
+        {
+          timevar_push (tv_xml);
+          print_xml ();
+          timevar_pop (tv_xml);
+        }
     }
 
   /* Stop if there were errors, to avoid trashing previous output
@@ -193,9 +200,12 @@ main (int argc, char *argv[])
   timevar_pop (tv_free);
 
   /* Output the tables and the parser to ftable.  In file output.  */
-  timevar_push (tv_parser);
-  output ();
-  timevar_pop (tv_parser);
+  if (generate)
+    {
+      timevar_push (tv_parser);
+      output ();
+      timevar_pop (tv_parser);
+    }
 
   timevar_push (tv_free);
   nullable_free ();
@@ -225,8 +235,6 @@ main (int argc, char *argv[])
   timevar_stop (tv_total);
   timevar_print (stderr);
 
-  cleanup_caret ();
-
   /* Fix input file now, even if there are errors: that's less
      warnings in the following runs.  */
   if (!fixits_empty ())
@@ -240,5 +248,7 @@ main (int argc, char *argv[])
     }
   uniqstrs_free ();
 
+  complain_free ();
+
   return complaint_status ? EXIT_FAILURE : EXIT_SUCCESS;
 }
