@@ -1,6 +1,6 @@
 /* Definitions for symtab.c and callers, part of Bison.
 
-   Copyright (C) 1984, 1989, 1992, 2000-2002, 2004-2015, 2018 Free
+   Copyright (C) 1984, 1989, 1992, 2000-2002, 2004-2015, 2018-2019 Free
    Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -40,7 +40,7 @@ typedef enum
 {
   unknown_sym,          /**< Undefined.  */
   token_sym,            /**< Terminal. */
-  nterm_sym             /**< Non-terminal. */
+  nterm_sym             /**< Nonterminal. */
 } symbol_class;
 
 
@@ -72,7 +72,7 @@ typedef enum
     needed,
     /** Defined with %type or %token (good).  */
     declared,
-  } status;
+  } declaration_status;
 
 enum code_props_type
   {
@@ -122,7 +122,7 @@ struct sym_content
   uniqstr type_name;
 
   /** Its \c \%type's location.  */
-  location type_location;
+  location type_loc;
 
   /** Any \c \%destructor (resp. \%printer) declared specificially for this
       symbol.
@@ -134,13 +134,17 @@ struct sym_content
   code_props props[CODE_PROPS_SIZE];
 
   symbol_number number;
-  location prec_location;
+  location prec_loc;
   int prec;
   assoc assoc;
+
+  /** The user specified token number.
+
+      E.g., %token FOO 42.*/
   int user_token_number;
 
   symbol_class class;
-  status status;
+  declaration_status status;
 };
 
 /** Undefined user number.  */
@@ -212,7 +216,10 @@ code_props *symbol_code_props_get (symbol *sym, code_props_type kind);
     Do nothing if invoked with \c undef_assoc as \c assoc.  */
 void symbol_precedence_set (symbol *sym, int prec, assoc a, location loc);
 
-/** Set the \c class associated with \c sym.  */
+/** Set the \c class associated with \c sym.
+
+    Whether \c declaring means whether this class definition comes
+    from %nterm or %token.  */
 void symbol_class_set (symbol *sym, symbol_class class, location loc,
                        bool declaring);
 
@@ -239,8 +246,10 @@ extern symbol *accept;
 /** The user start symbol. */
 extern symbol *startsymbol;
 /** The location of the \c \%start declaration.  */
-extern location startsymbol_location;
+extern location startsymbol_loc;
 
+/** Whether a symbol declared with a type tag.  */
+extern bool tag_seen;
 
 
 /*-------------------.
@@ -312,7 +321,7 @@ typedef struct {
 
   /** Its status : "undeclared", "used" or "declared".
       It cannot be "needed".  */
-  status status;
+  declaration_status status;
 
   /** Any \c %destructor and %printer declared for this
       semantic type.  */
